/*
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Five flip/flop channels depending on pulse variation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f509.h>
#define _XTAL_FREQ  4000000     			// oscillator frequency for _delay()

#define TEMPO 20

 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);

unsigned char cnt, tempo, sequence, prev;
bit sGP0, sGP1, sGP2, sGP4, sGP5;

void decode_sequence (unsigned char last_value);

/* ************************************************************************* */
main (void)
{
	OPTION = ~T0CS & ~PSA & (nGPWU | nGPPU | T0SE | 0x2);	/*
	internal cycle clock
	prescaler assigned to timer 
	wake up disabled
	pull up disabled
	high to low source edge
	prescaler values:	+-----+-------+
						| 0x0 | 1:2   |
						| 0x1 | 1:4   |
						| 0x2 | 1:8   | <---
						| 0x3 | 1:16  |
						| 0x4 | 1:32  |
						| 0x5 | 1:64  |
						| 0x6 | 1:128 |
						| 0x7 | 1:256 |
						+-----+-------+	*/
	TRIS = 0b001000;						// GP3 as input, all other as output
	GPIO = 0;								// all ports off
	sGP0 = sGP1 = sGP2 = sGP4 = sGP5 = 0;
	tempo = TEMPO;
	sequence = 0;
	prev = 2;
	for (cnt = 0; cnt < 5; cnt++) {			// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
/* ************************************************************************* */
	for (;;) {								// loop for ever
		do {								// wait pulse
		} while (!GP3);						// pulse received

		TMR0 = 255 - 106;					// wait at least 106 * 8 = 848 usec
		do {} while (TMR0 >= 20);
		do {} while (GP3);					// wait end of pulse
		cnt = TMR0 + 1;						// TMR0 is now between 16 and 144, neutral is 80
		if (cnt < 70)						// 1250 usec
			decode_sequence (0);
		else if (cnt > 90)					// 1750 usec
			decode_sequence (1);
		GP0 = sGP0;							// set port 0
		GP1 = sGP1;							// set port 1
		GP2 = sGP2;							// set port 2
		GP4 = sGP4;							// set port 4
		GP5 = sGP5;							// set port 5
	}
}

/* ************************************************************************* */
void decode_sequence (unsigned char last_value)
{
	if (prev <= 1) {						// initial value
		if (prev ^= last_value) {
			sequence++;						// each time the value is changed, the sequence number is incremented
			tempo = TEMPO;
		}
		else {								// when the value doesn't change, 
			if (tempo)
				tempo--;					// wait some milliseconds before accepting the command
			else if (sequence) {
				switch (sequence) {			// set ports according to the sequence number
				case 6:
					sGP0 = ~sGP0;			// toggle port 0
					break;
				case 7:
					sGP1 = ~sGP1;			// toggle port 1
					break;
				case 8:
					sGP2 = ~sGP2;			// toggle port 2
					break;
				case 9:
					sGP4 = ~sGP4;			// toggle port 4
					break;
				case 10:
					sGP5 = ~sGP5;			// toggle port 5
					break;
				}
				sequence = 0;
				tempo = TEMPO;
			}
		}
	}
	prev = last_value;
}
/* ************************************************************************* */
